#include <iostream>
#include <cstdio>
#include "fparser.hh"
#include <derivative.h>

using namespace std;
const double h=0.0001;

//char fn[100],deriv[100];

/*************** Functions for 2D ********************/


FunctionParser fp;

FunctionParser fp1;

double deriv_num2D(double pt)
{double res;
double p1=pt+2*h,p2=pt+h,p3=pt-h,p4=pt-2*h;
res=(-1*(fp.Eval(&p1))+8*(fp.Eval(&p2))-8*(fp.Eval(&p3))+fp.Eval(&p4))/(12*h);   //central difference formula of order four
 return res;
}

char* eq_tangent(double pt)                           //pass x coordinate
{
     char* eq = new char[100];
    double slope=deriv_num2D(pt);
    double y=fp.Eval(&pt);
    double cons=y-pt*slope;
    sprintf(eq,"y = %.2f*x + %.2f",slope,cons);
    return eq;
}

char* eq_normal(double pt)                            //pass x coordinate
{

    char* eq=new char[100];
    double slope=(-1)/(deriv_num2D(pt));
    double y=fp.Eval(&pt);
    double cons=y-pt*slope;
    sprintf(eq,"y = %.2f*x + %.2f",slope,cons);
    return eq;
}


/*************** Functions for 3D ********************/

char* symbolic_gradient(char * f)        //for symbolic gradient
{ char* grad[2];
  char* t=new char[100];
  grad[0]=deriv_fn(f,"x");                //make deriv_fn available here to get this working
  grad[1]=deriv_fn(f,"y");
  sprintf(t,"%si + %sj",grad[0],grad[1]);

  return t;
}

double* grad_num3D(double pt[2])
{
double *res=new double[2];

double p1[2]={pt[0]+2*h,pt[1]};
double p2[2]={pt[0]+h,pt[1]};
double p3[2]={pt[0]-h,pt[1]};
double p4[2]={pt[0]-2*h,pt[1]};

res[0]=(-1*(fp1.Eval(p1))+8*(fp1.Eval(p2))-8*(fp1.Eval(p3))+fp1.Eval(p4))/(12*h);   //central difference formula of order four

p1[0]=p2[0]=p3[0]=p4[0]=pt[0];
p1[1]=pt[1]+2*h;
p2[1]=pt[1]+h;
p3[1]=pt[1]-h;
p4[1]=pt[1]-2*h;

res[1]=(-1*(fp1.Eval(p1))+8*(fp1.Eval(p2))-8*(fp1.Eval(p3))+fp1.Eval(p4))/(12*h);   //central difference formula of order four

 return res;
}


double dir_der(double vars[2],double dirn[2])            //vars is a 1D array with 2 elements:x,y coordinate
{                                                    //dirn is a 1D array with 2 ele v,u which give direction
double *grad=grad_num3D(vars);
double l=sqrt(pow(dirn[0],2)+pow(dirn[1],2));
double res=grad[0]*dirn[0]/l+grad[1]*dirn[1]/l;
delete[] grad;
return res;
}

char* eq_normal3D(double* vars)                           //vars is an array with 2 elements:x,y coordinate
{
    double zval=fp1.Eval(vars);
    char* eq=new char[100];

    double* FxFy=grad_num3D(vars);
    double Fx=FxFy[0],Fy=FxFy[1];

    delete[] FxFy;
    sprintf(eq,"(%.2f-x)/%.2f=(%.2f-y)/%.2f=(z-(%.2f))",vars[0],Fx,vars[1],Fy,zval);
    return eq;
}
char* eq_TangentPlane(double* vars)                     //vars is an array with 2 elements:x,y coordinate
{
    double zval=fp1.Eval(vars);
    char* eq=new char[100];

    double* FxFy=grad_num3D(vars);
    double Fx=FxFy[0],Fy=FxFy[1];

    delete[] FxFy;

    double cons=zval-Fx*vars[0]-Fy*vars[1];

    sprintf(eq,"z=(%.2f)*x+(%.2f)*y+(%.2f)",Fx,Fy,cons);
    return eq;
}
/*****************************************************/
char* D3(char* fn,double x,double y,char ch)
{
fp1.Parse(fn,"x,y");
double vars[2];
char* t;

vars[0]=x;
vars[1]=y;
double *grad;

switch(ch)
{
    case 'g' : grad=grad_num3D(vars);
               t=new char[50];
               sprintf(t,"%.2fi + %.2fj",grad[0],grad[1]);
               break;

    case 't' : t=eq_TangentPlane(vars);
                break;

    case 'n' : t=eq_normal3D(vars);
                break;

}


return t;

}

char* D3(char* fn,double x,double y,double d1,double d2)
{
    fp1.Parse(fn,"x,y");
    double d[2];
    d[0]=d1;
    d[1]=d2;

    double vars[2];
    vars[0]=x;
    vars[1]=y;

    double res=dir_der(vars,d);
    char* t =new char[50];
    sprintf(t,"%f",res);
    return t;
}

char* D2(char* fn,double x,char ch)
{
    fp.Parse(fn,"x");
    double slope;
    char* t;
    switch(ch)
    {
        case 'd' :  slope=deriv_num2D(x);
                    t=new char[50];
                    sprintf(t,"%f",slope);
                    break;

        case 't' :   t=eq_tangent(x);
                     break;

        case 'n' :   t=eq_normal(x);
                     break;
    }
return t;



}

char* D3(char* fn)
{
    return symbolic_gradient(fn);
}


